@@ -18,11 +18,10 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-@bug SMS Dialog issues on Hildon
+@bug Need to add unit tests
 @bug Session timeouts are bad, possible solutions:
 	@li For every X minutes, if logged in, attempt login
 	@li Restructure so code handling login/dial/sms is beneath everything else and login attempts are made if those fail
-@todo Force login on connect if not already done
 @todo Can't text from dialpad (so can't do any arbitrary number texts)
 @todo Add logging support to make debugging issues for people a lot easier
 """
@@ -38,7 +37,6 @@ import base64
 import ConfigParser
 import itertools
 import warnings
-import traceback
 
 import gtk
 import gtk.glade
@@ -48,6 +46,7 @@ try:
 except ImportError:
 	hildon = None
 
+import constants
 import gtk_toolbox
 
 
@@ -69,11 +68,6 @@ def display_error_message(msg):
 
 class Dialcentral(object):
 
-	__pretty_app_name__ = "DialCentral"
-	__app_name__ = "dialcentral"
-	__version__ = "0.9.6"
-	__app_magic__ = 0xdeadbeef
-
 	_glade_files = [
 		'/usr/lib/dialcentral/dialcentral.glade',
 		os.path.join(os.path.dirname(__file__), "dialcentral.glade"),
@@ -155,7 +149,7 @@ class Dialcentral(object):
 		if hildon is not None:
 			self._window.set_title("Keypad")
 		else:
-			self._window.set_title("%s - Keypad" % self.__pretty_app_name__)
+			self._window.set_title("%s - Keypad" % constants.__pretty_app_name__)
 
 		callbackMapping = {
 			"on_dialpad_quit": self._on_close,
@@ -200,7 +194,7 @@ class Dialcentral(object):
 			osso = None
 		self._osso = None
 		if osso is not None:
-			self._osso = osso.Context(Dialcentral.__app_name__, Dialcentral.__version__, False)
+			self._osso = osso.Context(constants.__app_name__, constants.__version__, False)
 			device = osso.DeviceState(self._osso)
 			device.set_device_state_callback(self._on_device_state_change, 0)
 		else:
@@ -214,7 +208,7 @@ class Dialcentral(object):
 		self._connection = None
 		if conic is not None:
 			self._connection = conic.Connection()
-			self._connection.connect("connection-event", self._on_connection_change, Dialcentral.__app_magic__)
+			self._connection.connect("connection-event", self._on_connection_change, constants.__app_magic__)
 			self._connection.request_connection(conic.CONNECT_FLAG_NONE)
 		else:
 			pass # warnings.warn("No Internet Connectivity API ", UserWarning)
@@ -284,10 +278,10 @@ class Dialcentral(object):
 		fsContactsPath = os.path.join(self._data_path, "contacts")
 		fileBackend = file_backend.FilesystemAddressBookFactory(fsContactsPath)
 		for backendId in (self.GV_BACKEND, self.GC_BACKEND):
-			self._dialpads[backendId].dial = self._on_dial_clicked
-			self._recentViews[backendId].number_selected = self._on_number_selected
-			self._messagesViews[backendId].number_selected = self._on_number_selected
-			self._contactsViews[backendId].number_selected = self._on_number_selected
+			self._dialpads[backendId].number_selected = self._select_action
+			self._recentViews[backendId].number_selected = self._select_action
+			self._messagesViews[backendId].number_selected = self._select_action
+			self._contactsViews[backendId].number_selected = self._select_action
 
 			addressBooks = [
 				self._phoneBackends[backendId],
@@ -317,89 +311,123 @@ class Dialcentral(object):
 
 		self.attempt_login(2)
 
-		return False
-
-	def attempt_login(self, numOfAttempts = 10):
+	def attempt_login(self, numOfAttempts = 10, force = False):
 		"""
 		@todo Handle user notification better like attempting to login and failed login
 
-		@note Not meant to be called directly, but run as a seperate thread.
+		@note This must be run outside of the UI lock
 		"""
-		assert 0 < numOfAttempts, "That was pointless having 0 or less login attempts"
+		try:
+			assert 0 <= numOfAttempts, "That was pointless having 0 or less login attempts"
+			assert self._initDone, "Attempting login before app is fully loaded"
+			if not self._deviceIsOnline:
+				raise RuntimeError("Unable to login, device is not online")
 
+			serviceId = self.NULL_BACKEND
+			loggedIn = False
+			if not force:
+				try:
+					self.refresh_session()
+					serviceId = self._defaultBackendId
+					loggedIn = True
+				except StandardError, e:
+					warnings.warn('Session refresh failed with the following message "%s"' % e.message, UserWarning, 2)
+
+			if not loggedIn:
+				loggedIn, serviceId = self._login_by_user(numOfAttempts)
+
+			with gtk_toolbox.gtk_lock():
+				self._change_loggedin_status(serviceId)
+		except StandardError, e:
+			with gtk_toolbox.gtk_lock():
+				self._errorDisplay.push_exception(e)
+
+	def refresh_session(self):
+		"""
+		@note Thread agnostic
+		"""
+		assert self._initDone, "Attempting login before app is fully loaded"
 		if not self._deviceIsOnline:
-			warnings.warn("Attempted to login while device was offline")
-			return False
-		elif self._phoneBackends is None or len(self._phoneBackends) < len(self.BACKENDS):
-			warnings.warn(
-				"Attempted to login before initialization is complete, did an event fire early?"
-			)
-			return False
+			raise RuntimeError("Unable to login, device is not online")
 
 		loggedIn = False
-		try:
-			username, password = self._credentials
-			serviceId = self._defaultBackendId
+		if not loggedIn:
+			loggedIn = self._login_by_cookie()
+		if not loggedIn:
+			loggedIn = self._login_by_settings()
 
-			# Attempt using the cookies
-			loggedIn = self._phoneBackends[self._defaultBackendId].is_authed()
-			if loggedIn:
-				warnings.warn(
-					"Logged into %r through cookies" % self._phoneBackends[self._defaultBackendId],
-					UserWarning, 2
-				)
+		if not loggedIn:
+			raise RuntimeError("Login Failed")
 
-			# Attempt using the settings file
-			if not loggedIn and username and password:
-				loggedIn = self._phoneBackends[self._defaultBackendId].login(username, password)
-				if loggedIn:
-					warnings.warn(
-						"Logged into %r through settings" % self._phoneBackends[self._defaultBackendId],
-						UserWarning, 2
-					)
-
-			# Query the user for credentials
-			for attemptCount in xrange(numOfAttempts):
-				if loggedIn:
-					break
-				with gtk_toolbox.gtk_lock():
-					availableServices = {
-						self.GV_BACKEND: "Google Voice",
-						self.GC_BACKEND: "Grand Central",
-					}
-					credentials = self._credentialsDialog.request_credentials_from(
-						availableServices, defaultCredentials = self._credentials
-					)
-					serviceId, username, password = credentials
-
-				loggedIn = self._phoneBackends[serviceId].login(username, password)
-			if 0 < attemptCount:
-				warnings.warn(
-					"Logged into %r through user request" % self._phoneBackends[serviceId],
-					UserWarning, 2
-				)
-		except RuntimeError, e:
-			warnings.warn(traceback.format_exc())
-			self._errorDisplay.push_exception_with_lock(e)
-
-		with gtk_toolbox.gtk_lock():
-			if loggedIn:
-				self._credentials = username, password
-				self._change_loggedin_status(serviceId)
-			else:
-				self._errorDisplay.push_message("Login Failed")
-				self._change_loggedin_status(self.NULL_BACKEND)
+	def _login_by_cookie(self):
+		"""
+		@note Thread agnostic
+		"""
+		loggedIn = self._phoneBackends[self._defaultBackendId].is_authed()
+		if loggedIn:
+			warnings.warn(
+				"Logged into %r through cookies" % self._phoneBackends[self._defaultBackendId],
+				UserWarning, 2
+			)
 		return loggedIn
 
-	def _on_close(self, *args, **kwds):
-		try:
-			if self._osso is not None:
-				self._osso.close()
+	def _login_by_settings(self):
+		"""
+		@note Thread agnostic
+		"""
+		username, password = self._credentials
+		loggedIn = self._phoneBackends[self._defaultBackendId].login(username, password)
+		if loggedIn:
+			self._credentials = username, password
+			warnings.warn(
+				"Logged into %r through settings" % self._phoneBackends[self._defaultBackendId],
+				UserWarning, 2
+			)
+		return loggedIn
 
-			if self._initDone:
-				self._save_settings()
-		finally:
-			gtk.main_quit()
+	def _login_by_user(self, numOfAttempts):
+		"""
+		@note This must be run outside of the UI lock
+		"""
+		loggedIn, (username, password) = False, self._credentials
+		tmpServiceId = self.NULL_BACKEND
+		for attemptCount in xrange(numOfAttempts):
+			if loggedIn:
+				break
+			availableServices = {
+				self.GV_BACKEND: "Google Voice",
+				self.GC_BACKEND: "Grand Central",
+			}
+			with gtk_toolbox.gtk_lock():
+				credentials = self._credentialsDialog.request_credentials_from(
+					availableServices, defaultCredentials = self._credentials
+				)
+			tmpServiceId, username, password = credentials
+			loggedIn = self._phoneBackends[tmpServiceId].login(username, password)
+
+		if loggedIn:
+			serviceId = tmpServiceId
+			self._credentials = username, password
+			warnings.warn(
+				"Logged into %r through user request" % self._phoneBackends[serviceId],
+				UserWarning, 2
+			)
+		else:
+			serviceId = self.NULL_BACKEND
+
+		return loggedIn, serviceId
+
+	def _select_action(self, action, number, message):
+		self.refresh_session()
+		if action == "select":
+			self._dialpads[self._selectedBackendId].set_number(number)
+			self._notebook.set_current_page(self.KEYPAD_TAB)
+		elif action == "dial":
+			self._on_dial_clicked(number)
+		elif action == "sms":
+			self._on_sms_clicked(number, message)
+		else:
+			assert False, "Unknown action: %s" % action
 
 	def _change_loggedin_status(self, newStatus):
 		oldStatus = self._selectedBackendId
@@ -428,10 +456,10 @@ class Dialcentral(object):
 		"""
 		@note UI Thread
 		"""
-		self._defaultBackendId = int(config.get(self.__pretty_app_name__, "active"))
 		try:
+			self._defaultBackendId = int(config.get(constants.__pretty_app_name__, "active"))
 			blobs = (
-				config.get(self.__pretty_app_name__, "bin_blob_%i" % i)
+				config.get(constants.__pretty_app_name__, "bin_blob_%i" % i)
 				for i in xrange(len(self._credentials))
 			)
 			creds = (
@@ -471,11 +499,11 @@ class Dialcentral(object):
 		"""
 		@note Thread Agnostic
 		"""
-		config.add_section(self.__pretty_app_name__)
-		config.set(self.__pretty_app_name__, "active", str(self._selectedBackendId))
+		config.add_section(constants.__pretty_app_name__)
+		config.set(constants.__pretty_app_name__, "active", str(self._selectedBackendId))
 		for i, value in enumerate(self._credentials):
 			blob = base64.b64encode(value)
-			config.set(self.__pretty_app_name__, "bin_blob_%i" % i, blob)
+			config.set(constants.__pretty_app_name__, "bin_blob_%i" % i, blob)
 		for backendId, view in itertools.chain(
 			self._dialpads.iteritems(),
 			self._accountViews.iteritems(),
@@ -504,6 +532,16 @@ class Dialcentral(object):
 		with open(self._user_settings, "wb") as configFile:
 			config.write(configFile)
 
+	def _on_close(self, *args, **kwds):
+		try:
+			if self._osso is not None:
+				self._osso.close()
+
+			if self._initDone:
+				self._save_settings()
+		finally:
+			gtk.main_quit()
+
 	def _on_device_state_change(self, shutdown, save_unsaved_data, memory_low, system_inactivity, message, userData):
 		"""
 		For shutdown or save_unsaved_data, our only state is cookies and I think the cookie manager handles that for us.
@@ -570,7 +608,7 @@ class Dialcentral(object):
 		self._contactsViews[self._selectedBackendId].clear()
 		self._change_loggedin_status(self.NULL_BACKEND)
 
-		backgroundLogin = threading.Thread(target=self.attempt_login, args=[2])
+		backgroundLogin = threading.Thread(target=self.attempt_login, args=[2, True])
 		backgroundLogin.setDaemon(True)
 		backgroundLogin.start()
 
@@ -588,23 +626,9 @@ class Dialcentral(object):
 		if hildon is not None:
 			self._window.set_title(tabTitle)
 		else:
-			self._window.set_title("%s - %s" % (self.__pretty_app_name__, tabTitle))
-
-	def _on_number_selected(self, action, number, message):
-		if action == "select":
-			self._dialpads[self._selectedBackendId].set_number(number)
-			self._notebook.set_current_page(self.KEYPAD_TAB)
-		elif action == "dial":
-			self._on_dial_clicked(number)
-		elif action == "sms":
-			self._on_sms_clicked(number, message)
-		else:
-			assert False, "Unknown action: %s" % action
+			self._window.set_title("%s - %s" % (constants.__pretty_app_name__, tabTitle))
 
 	def _on_sms_clicked(self, number, message):
-		"""
-		@todo Potential blocking on web access, maybe we should defer parts of this or put up a dialog?
-		"""
 		assert number
 		assert message
 		try:
@@ -630,9 +654,6 @@ class Dialcentral(object):
 			self._errorDisplay.push_exception(e)
 
 	def _on_dial_clicked(self, number):
-		"""
-		@todo Potential blocking on web access, maybe we should defer parts of this or put up a dialog?
-		"""
 		assert number
 		try:
 			loggedIn = self._phoneBackends[self._selectedBackendId].is_authed()
@@ -675,8 +696,8 @@ class Dialcentral(object):
 
 	def _on_about_activate(self, *args):
 		dlg = gtk.AboutDialog()
-		dlg.set_name(self.__pretty_app_name__)
-		dlg.set_version(self.__version__)
+		dlg.set_name(constants.__pretty_app_name__)
+		dlg.set_version(constants.__version__)
 		dlg.set_copyright("Copyright 2008 - LGPL")
 		dlg.set_comments("Dialer is designed to interface with your Google Grandcentral account.  This application is not affiliated with Google or Grandcentral in any way")
 		dlg.set_website("http://gc-dialer.garage.maemo.org/")
@@ -699,7 +720,7 @@ def run_doctest():
 def run_dialpad():
 	gtk.gdk.threads_init()
 	if hildon is not None:
-		gtk.set_application_name(Dialcentral.__pretty_app_name__)
+		gtk.set_application_name(constants.__pretty_app_name__)
 	handle = Dialcentral()
 	gtk.main()
 
