@@ -28,6 +28,7 @@ import gobject
 import gtk
 
 import gtk_toolbox
+import null_backend
 
 
 def make_ugly(prettynumber):
@@ -93,69 +94,6 @@ def make_pretty(phonenumber):
 	return prettynumber
 
 
-def make_idler(func):
-	"""
-	Decorator that makes a generator-function into a function that will continue execution on next call
-	"""
-	a = []
-
-	def decorated_func(*args, **kwds):
-		if not a:
-			a.append(func(*args, **kwds))
-		try:
-			a[0].next()
-			return True
-		except StopIteration:
-			del a[:]
-			return False
-
-	decorated_func.__name__ = func.__name__
-	decorated_func.__doc__ = func.__doc__
-	decorated_func.__dict__.update(func.__dict__)
-
-	return decorated_func
-
-
-class DummyAddressBook(object):
-	"""
-	Minimal example of both an addressbook factory and an addressbook
-	"""
-
-	def clear_caches(self):
-		pass
-
-	def get_addressbooks(self):
-		"""
-		@returns Iterable of (Address Book Factory, Book Id, Book Name)
-		"""
-		yield self, "", "None"
-
-	def open_addressbook(self, bookId):
-		return self
-
-	@staticmethod
-	def contact_source_short_name(contactId):
-		return ""
-
-	@staticmethod
-	def factory_name():
-		return ""
-
-	@staticmethod
-	def get_contacts():
-		"""
-		@returns Iterable of (contact id, contact name)
-		"""
-		return []
-
-	@staticmethod
-	def get_contact_details(contactId):
-		"""
-		@returns Iterable of (Phone Type, Phone Number)
-		"""
-		return []
-
-
 class MergedAddressBook(object):
 	"""
 	Merger of all addressbooks
@@ -519,7 +457,7 @@ class Dialpad(object):
 	def disable(self):
 		pass
 
-	def dial(self, number):
+	def number_selected(self, action, number, message):
 		"""
 		@note Actual dial function is patched in later
 		"""
@@ -556,7 +494,10 @@ class Dialpad(object):
 		pass
 
 	def _on_dial_clicked(self, widget):
-		self.dial(self.get_number())
+		action = PhoneTypeSelector.ACTION_DIAL
+		phoneNumber = self.get_number()
+		message = ""
+		self.number_selected(action, phoneNumber, message)
 
 	def _on_clear_number(self, *args):
 		self.clear()
@@ -663,9 +604,6 @@ class AccountInfo(object):
 		self._callbackCombo.get_child().set_text(make_pretty(callbackNumber))
 
 	def _set_callback_number(self, number):
-		"""
-		@todo Potential blocking on web access, maybe we should defer this or put up a dialog?
-		"""
 		try:
 			if not self._backend.is_valid_syntax(number):
 				self._errorDisplay.push_message("%s is not a valid callback number" % number)
@@ -678,31 +616,50 @@ class AccountInfo(object):
 			self._errorDisplay.push_exception(e)
 
 	def _on_callbackentry_changed(self, *args):
-		"""
-		@todo Potential blocking on web access, maybe we should defer this or put up a dialog?
-		"""
 		text = self.get_selected_callback_number()
 		self._set_callback_number(text)
 
 
 class RecentCallsView(object):
 
+	NUMBER_IDX = 0
+	DATE_IDX = 1
+	ACTION_IDX = 2
+	FROM_IDX = 3
+
 	def __init__(self, widgetTree, backend, errorDisplay):
 		self._errorDisplay = errorDisplay
 		self._backend = backend
 
 		self._isPopulated = False
-		self._recentmodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
+		self._recentmodel = gtk.ListStore(
+			gobject.TYPE_STRING, # number
+			gobject.TYPE_STRING, # date
+			gobject.TYPE_STRING, # action
+			gobject.TYPE_STRING, # from
+		)
 		self._recentview = widgetTree.get_widget("recentview")
 		self._recentviewselection = None
 		self._onRecentviewRowActivatedId = 0
 
-		# @todo Make seperate columns for each item in recent item payload
 		textrenderer = gtk.CellRendererText()
-		self._recentviewColumn = gtk.TreeViewColumn("Calls")
-		self._recentviewColumn.pack_start(textrenderer, expand=True)
-		self._recentviewColumn.add_attribute(textrenderer, "text", 1)
-		self._recentviewColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+		textrenderer.set_property("yalign", 0)
+		self._dateColumn = gtk.TreeViewColumn("Date")
+		self._dateColumn.pack_start(textrenderer, expand=True)
+		self._dateColumn.add_attribute(textrenderer, "text", self.DATE_IDX)
+
+		textrenderer = gtk.CellRendererText()
+		textrenderer.set_property("yalign", 0)
+		self._actionColumn = gtk.TreeViewColumn("Action")
+		self._actionColumn.pack_start(textrenderer, expand=True)
+		self._actionColumn.add_attribute(textrenderer, "text", self.ACTION_IDX)
+
+		textrenderer = gtk.CellRendererText()
+		textrenderer.set_property("yalign", 0)
+		self._fromColumn = gtk.TreeViewColumn("From")
+		self._fromColumn.pack_start(textrenderer, expand=True)
+		self._fromColumn.add_attribute(textrenderer, "text", self.FROM_IDX)
+		self._fromColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 
 		self._window = gtk_toolbox.find_parent_window(self._recentview)
 		self._phoneTypeSelector = PhoneTypeSelector(widgetTree, self._backend)
@@ -711,7 +668,9 @@ class RecentCallsView(object):
 		assert self._backend.is_authed()
 		self._recentview.set_model(self._recentmodel)
 
-		self._recentview.append_column(self._recentviewColumn)
+		self._recentview.append_column(self._dateColumn)
+		self._recentview.append_column(self._actionColumn)
+		self._recentview.append_column(self._fromColumn)
 		self._recentviewselection = self._recentview.get_selection()
 		self._recentviewselection.set_mode(gtk.SELECTION_SINGLE)
 
@@ -722,7 +681,9 @@ class RecentCallsView(object):
 
 		self.clear()
 
-		self._recentview.remove_column(self._recentviewColumn)
+		self._recentview.remove_column(self._dateColumn)
+		self._recentview.remove_column(self._actionColumn)
+		self._recentview.remove_column(self._fromColumn)
 		self._recentview.set_model(None)
 
 	def number_selected(self, action, number, message):
@@ -766,9 +727,11 @@ class RecentCallsView(object):
 			self._isPopulated = False
 			recentItems = []
 
-		for personsName, phoneNumber, date, action in recentItems:
-			description = "%s on %s from/to %s - %s" % (action.capitalize(), date, personsName, phoneNumber)
-			item = (phoneNumber, description)
+		for personName, phoneNumber, date, action in recentItems:
+			if not personName:
+				personName = "Unknown"
+			description = "%s (%s)" % (phoneNumber, personName)
+			item = (phoneNumber, date, action.capitalize(), description)
 			with gtk_toolbox.gtk_lock():
 				self._recentmodel.append(item)
 
@@ -779,10 +742,10 @@ class RecentCallsView(object):
 		if not itr:
 			return
 
-		number = self._recentmodel.get_value(itr, 0)
+		number = self._recentmodel.get_value(itr, self.NUMBER_IDX)
 		number = make_ugly(number)
 		contactPhoneNumbers = [("Phone", number)]
-		description = self._recentmodel.get_value(itr, 1)
+		description = self._recentmodel.get_value(itr, self.FROM_IDX)
 
 		action, phoneNumber, message = self._phoneTypeSelector.run(
 			contactPhoneNumbers,
@@ -799,22 +762,44 @@ class RecentCallsView(object):
 
 class MessagesView(object):
 
+	NUMBER_IDX = 0
+	DATE_IDX = 1
+	HEADER_IDX = 2
+	MESSAGE_IDX = 3
+
 	def __init__(self, widgetTree, backend, errorDisplay):
 		self._errorDisplay = errorDisplay
 		self._backend = backend
 
 		self._isPopulated = False
-		self._messagemodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
+		self._messagemodel = gtk.ListStore(
+			gobject.TYPE_STRING, # number
+			gobject.TYPE_STRING, # date
+			gobject.TYPE_STRING, # header
+			gobject.TYPE_STRING, # message
+		)
 		self._messageview = widgetTree.get_widget("messages_view")
 		self._messageviewselection = None
 		self._onMessageviewRowActivatedId = 0
 
 		textrenderer = gtk.CellRendererText()
-		# @todo Make seperate columns for each item in message payload
-		self._messageviewColumn = gtk.TreeViewColumn("Messages")
-		self._messageviewColumn.pack_start(textrenderer, expand=True)
-		self._messageviewColumn.add_attribute(textrenderer, "markup", 1)
-		self._messageviewColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+		textrenderer.set_property("yalign", 0)
+		self._dateColumn = gtk.TreeViewColumn("Date")
+		self._dateColumn.pack_start(textrenderer, expand=True)
+		self._dateColumn.add_attribute(textrenderer, "markup", self.DATE_IDX)
+
+		textrenderer = gtk.CellRendererText()
+		textrenderer.set_property("yalign", 0)
+		self._headerColumn = gtk.TreeViewColumn("From")
+		self._headerColumn.pack_start(textrenderer, expand=True)
+		self._headerColumn.add_attribute(textrenderer, "markup", self.HEADER_IDX)
+
+		textrenderer = gtk.CellRendererText()
+		textrenderer.set_property("yalign", 0)
+		self._messageColumn = gtk.TreeViewColumn("Messages")
+		self._messageColumn.pack_start(textrenderer, expand=True)
+		self._messageColumn.add_attribute(textrenderer, "markup", self.MESSAGE_IDX)
+		self._messageColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 
 		self._window = gtk_toolbox.find_parent_window(self._messageview)
 		self._phoneTypeSelector = PhoneTypeSelector(widgetTree, self._backend)
@@ -823,7 +808,9 @@ class MessagesView(object):
 		assert self._backend.is_authed()
 		self._messageview.set_model(self._messagemodel)
 
-		self._messageview.append_column(self._messageviewColumn)
+		self._messageview.append_column(self._dateColumn)
+		self._messageview.append_column(self._headerColumn)
+		self._messageview.append_column(self._messageColumn)
 		self._messageviewselection = self._messageview.get_selection()
 		self._messageviewselection.set_mode(gtk.SELECTION_SINGLE)
 
@@ -834,7 +821,9 @@ class MessagesView(object):
 
 		self.clear()
 
-		self._messageview.remove_column(self._messageviewColumn)
+		self._messageview.remove_column(self._dateColumn)
+		self._messageview.remove_column(self._headerColumn)
+		self._messageview.remove_column(self._messageColumn)
 		self._messageview.set_model(None)
 
 	def number_selected(self, action, number, message):
@@ -880,7 +869,7 @@ class MessagesView(object):
 
 		for header, number, relativeDate, message in messageItems:
 			number = make_ugly(number)
-			row = (number, message)
+			row = (number, relativeDate, header, message)
 			with gtk_toolbox.gtk_lock():
 				self._messagemodel.append(row)
 
@@ -891,8 +880,8 @@ class MessagesView(object):
 		if not itr:
 			return
 
-		contactPhoneNumbers = [("Phone", self._messagemodel.get_value(itr, 0))]
-		description = self._messagemodel.get_value(itr, 1)
+		contactPhoneNumbers = [("Phone", self._messagemodel.get_value(itr, self.NUMBER_IDX))]
+		description = self._messagemodel.get_value(itr, self.MESSAGE_IDX)
 
 		action, phoneNumber, message = self._phoneTypeSelector.run(
 			contactPhoneNumbers,
@@ -914,7 +903,7 @@ class ContactsView(object):
 		self._backend = backend
 
 		self._addressBook = None
-		self._addressBookFactories = [DummyAddressBook()]
+		self._addressBookFactories = [null_backend.NullAddressBook()]
 
 		self._booksList = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
 		self._booksSelectionBox = widgetTree.get_widget("addressbook_combo")
