@@ -298,8 +298,6 @@ class GVDialer(object):
 			warnings.warn(traceback.format_exc())
 			raise RuntimeError("%s is not accesible" % self._setforwardURL)
 
-		# @bug This does not seem to be keeping on my tablet (but works on the
-		# desktop), or the reading isn't working too well
 		self._browser.cookies.save()
 		return True
 
@@ -447,9 +445,10 @@ class GVDialer(object):
 		self._token = tokenGroup.group(1)
 
 		anGroup = self._accountNumRe.search(page)
-		if anGroup is None:
-			raise RuntimeError("Could not extract account number from GoogleVoice")
-		self._accountNum = anGroup.group(1)
+		if anGroup is not None:
+			self._accountNum = anGroup.group(1)
+		else:
+			warnings.warn("Could not extract account number from GoogleVoice", UserWarning, 2)
 
 		self._callbackNumbers = {}
 		for match in self._callbackRe.finditer(page):
@@ -510,12 +509,12 @@ class GVDialer(object):
 	_relativeVoicemailTimeRegex = re.compile(r"""<span class="gc-message-relative">(.*?)</span>""", re.MULTILINE)
 	_voicemailNumberRegex = re.compile(r"""<input type="hidden" class="gc-text gc-quickcall-ac" value="(.*?)"/>""", re.MULTILINE)
 	_prettyVoicemailNumberRegex = re.compile(r"""<span class="gc-message-type">(.*?)</span>""", re.MULTILINE)
-	_voicemailLocationRegex = re.compile(r"""<span class="gc-message-location">(.*?)</span>""", re.MULTILINE)
+	_voicemailLocationRegex = re.compile(r"""<span class="gc-message-location">.*?<a.*?>(.*?)</a></span>""", re.MULTILINE)
 	_voicemailMessageRegex = re.compile(r"""<span class="gc-word-(.*?)">(.*?)</span>""", re.MULTILINE)
 
 	def _parse_voicemail(self, voicemailHtml):
 		splitVoicemail = self._seperateVoicemailsRegex.split(voicemailHtml)
-		for id, messageHtml in itergroup(splitVoicemail[1:], 2):
+		for messageId, messageHtml in itergroup(splitVoicemail[1:], 2):
 			exactTimeGroup = self._exactVoicemailTimeRegex.search(messageHtml)
 			exactTime = exactTimeGroup.group(1).strip() if exactTimeGroup else ""
 			exactTime = datetime.datetime.strptime(exactTime, "%m/%d/%y %I:%M %p")
@@ -536,7 +535,7 @@ class GVDialer(object):
 			) if messageGroups else ()
 
 			yield {
-				"id": id.strip(),
+				"id": messageId.strip(),
 				"time": exactTime,
 				"relTime": relativeTime,
 				"prettyNumber": prettyNumber,
@@ -568,7 +567,7 @@ class GVDialer(object):
 
 	def _parse_sms(self, smsHtml):
 		splitSms = self._seperateVoicemailsRegex.split(smsHtml)
-		for id, messageHtml in itergroup(splitSms[1:], 2):
+		for messageId, messageHtml in itergroup(splitSms[1:], 2):
 			exactTimeGroup = self._exactVoicemailTimeRegex.search(messageHtml)
 			exactTime = exactTimeGroup.group(1).strip() if exactTimeGroup else ""
 			exactTime = datetime.datetime.strptime(exactTime, "%m/%d/%y %I:%M %p")
@@ -590,7 +589,7 @@ class GVDialer(object):
 			messageParts = itertools.izip(fromParts, textParts, timeParts)
 
 			yield {
-				"id": id.strip(),
+				"id": messageId.strip(),
 				"time": exactTime,
 				"relTime": relativeTime,
 				"prettyNumber": prettyNumber,
@@ -619,7 +618,7 @@ def test_backend(username, password):
 	print "Authenticated: ", backend.is_authed()
 	print "Login?: ", backend.login(username, password)
 	print "Authenticated: ", backend.is_authed()
-	print "Token: ", backend._token
+	# print "Token: ", backend._token
 	print "Account: ", backend.get_account_number()
 	print "Callback: ", backend.get_callback_number()
 	# print "All Callback: ",
